"""Functions for scraping IMDB"""

from bs4 import BeautifulSoup as bs
import requests
import re


ROOT = "http://imdb.com"


def get_html(link):
    return bs(requests.get(link).text)


def get_a(html, find=''):
    """Finds all the 'a' tags with find in their href"""
    links = []
    for a in html.find_all('a'):
        if a.get('href').find(find) != -1:
            links.append(a)
    return links


def episode_list(a):
    """List of all episodes of a season"""
    html = get_html(ROOT + a.get('href'))
    div = html.find('div', {'class': "list detail eplist"})
    links = []
    for tag in div.find_all('a', {'itemprop': "name"}):
        links.append(tag)
    return links


def get_rating(html):
    """Rating of an episode
    Works for any page with a rating
    Incase no rating found, returns None
    """
    try:
        div = html.find('div', {'class': "titlePageSprite star-box-giga-star"})
        return div.text.strip(' ')
    except AttributeError:
        return None


def get_name_date(html):
    head = html.find('h1', {'class': "header"})
    spans = head.findChildren()
    name = spans[0].text
    if len(spans) == 2:
        date = spans[1].text[1:-1]
    else:
        date = None
    return name, date


def get_season_epi_num(html):
    head = html.find('h2', {'class': "tv_header"})
    spans = head.findChildren()
    if len(spans) == 2:
        season, epi = spans[1].text.split(',')
        season = re.search('\d+', season).group()
        epi = re.search('\d+', epi).group()
        return season, epi
    else:
        return None, None


def parse_episode(a):
    """Collects data related to an episode"""
    d = {}
    html = get_html(ROOT + a.get('href'))
    d['rating'] = get_rating(html)
    d['episode-name'], d['date'] = get_name_date(html)
    season, d['episode-num'] = get_season_epi_num(html)
    return season, d


def parse(link):
    """Parses a Tv Series
    returns the dataset as a dictionary
    """
    html = get_html(link)

    data = {'rating': get_rating(html),
            'name': get_name_date(html)[0]}

    div = html.find(id="title-episode-widget")
    season_tags = get_a(div, find="season=")
    episodes = {}
    for slink in season_tags:
        for e in episode_list(slink):
            season, d = parse_episode(e)
            if season in episodes:
                episodes[season].append(d)
            else:
                episodes[season] = [d]
    data['episodes'] = episodes
    return data



"""Helper functions for easy graph making"""

import matplotlib.pyplot as plt
import jsonhelper as jh


def graphdata(data):
    """returns ratings and episode number
    to be used for making graphs"""
    data = jh.get_ratings(data)
    num = 1
    rating_final = []
    episode_final = []
    for k,v in data.iteritems():
        rating=[]
        epinum=[]
        for r in v:
            if r != None:
                rating.append(float(r))
                epinum.append(num)
                num+=1
        rating_final.append(rating)
        episode_final.append(epinum)
    return rating_final,episode_final


def graph(data):
    """Draws graph of rating vs episode number"""
    title = data['name'] + ' (' + data['rating'] + ') '
    plt.title(title)
    plt.xlabel('Episode Number')
    plt.ylabel('Ratings')
    rf,ef=graphdata(data)
    col=['red', 'green' , 'orange']
    for i in range(len(rf)):
        x,y=ef[i],rf[i]
        k = i + 1
        plt.plot(x, y,color=col[i%3])
    x1, x2, y1, y2 = plt.axis()
    y2 = 10
    if y1 > 7:
        y1 = 7
    plt.axis([x1, x2, y1, y2])
    plt.show()



import json
import collections as co


def dict_to_json(json_dict):
    """
    convert from a dictionary to json
    """
    return json.dumps(json_dict)


def json_to_dict(json_str):
    """
    convert from json to dictionary
    """
    return json.loads(json_str)


def get_data(fname):
    """returns dictionary"""
    with open(fname) as f:
        return json_to_dict(f.read())


def collapse(season):
    """Collects all the ratings of episodes from a season"""
    l = []
    for epi in season:
        l.append(epi['rating'])
    return l


def get_ratings(data):
    """Ratings of all the episodes of all the seasons"""
    episodes = data['episodes']
    ratings = {}
    for season in episodes:
        ratings[season] = collapse(episodes[season])
    return co.OrderedDict(sorted(ratings.items()))



#!/usr/bin/env python2

from tvstats import __version__

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

setup(
    name = 'tvstats',
    version = __version__,
    author = 'Sartaj Singh',
    author_email = 'singhsartaj94@gmail.com',
    description = ('Scrape data of all the episodes of a Tv Series from IMDB'),
    long_description = open('README.rst').read() + '\n\n' + \
    open('CHANGELOG.rst').read(),
    license = 'MIT',
    keywords = 'tv tvseries series IMBD scrape crawl',
    url = 'http://github.com/leosartaj/tvstats',
    packages=find_packages(),
    scripts=['bin/tvstats'],
    install_requires = ['beautifulsoup4'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Environment :: Console',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
    ],
)
